//---------------------------------------------------------------------
// Math providers
//---------------------------------------------------------------------
"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var join_1 = require("../join");
var providers = require("./index");
var TotalFunctionConstraint = (function (_super) {
    __extends(TotalFunctionConstraint, _super);
    function TotalFunctionConstraint() {
        _super.apply(this, arguments);
    }
    // Proposes the return value of the total function as the value for the
    // proposed variable.
    TotalFunctionConstraint.prototype.resolveProposal = function (proposal, prefix) {
        var args = this.resolve(prefix).args;
        return [this.getReturnValue(args)];
    };
    // Check if our return is equivalent to the result of the total function.
    TotalFunctionConstraint.prototype.test = function (prefix) {
        var _a = this.resolve(prefix), args = _a.args, returns = _a.returns;
        return this.getReturnValue(args) === returns[0];
    };
    // Total functions always have a cardinality of 1
    TotalFunctionConstraint.prototype.getProposal = function (tripleIndex, proposed, prefix) {
        var proposal = this.proposalObject;
        proposal.providing = proposed;
        proposal.cardinality = 1;
        return proposal;
    };
    return TotalFunctionConstraint;
}(join_1.Constraint));
var TrigConstraint = (function (_super) {
    __extends(TrigConstraint, _super);
    function TrigConstraint() {
        _super.apply(this, arguments);
    }
    TrigConstraint.prototype.resolveTrigAttributes = function (args) {
        var degrees = args[0];
        var radians = args[1];
        //degrees which overrides radians. 
        if (!isNaN(degrees)) {
            radians = degreesToRadians(degrees);
        }
        return radians;
    };
    TrigConstraint.AttributeMapping = {
        "degrees": 0,
        "radians": 1
    };
    return TrigConstraint;
}(TotalFunctionConstraint));
var ValueOnlyConstraint = (function (_super) {
    __extends(ValueOnlyConstraint, _super);
    function ValueOnlyConstraint() {
        _super.apply(this, arguments);
    }
    ValueOnlyConstraint.AttributeMapping = {
        "value": 0
    };
    return ValueOnlyConstraint;
}(TotalFunctionConstraint));
function radiansToDegrees(radians) {
    return radians * (180 / Math.PI);
}
function degreesToRadians(degrees) {
    return degrees * (Math.PI / 180);
}
var Add = (function (_super) {
    __extends(Add, _super);
    function Add() {
        _super.apply(this, arguments);
    }
    Add.prototype.getReturnValue = function (args) {
        return args[0] + args[1];
    };
    return Add;
}(TotalFunctionConstraint));
var Subtract = (function (_super) {
    __extends(Subtract, _super);
    function Subtract() {
        _super.apply(this, arguments);
    }
    Subtract.prototype.getReturnValue = function (args) {
        return args[0] - args[1];
    };
    return Subtract;
}(TotalFunctionConstraint));
var Multiply = (function (_super) {
    __extends(Multiply, _super);
    function Multiply() {
        _super.apply(this, arguments);
    }
    Multiply.prototype.getReturnValue = function (args) {
        return args[0] * args[1];
    };
    return Multiply;
}(TotalFunctionConstraint));
var Divide = (function (_super) {
    __extends(Divide, _super);
    function Divide() {
        _super.apply(this, arguments);
    }
    Divide.prototype.getReturnValue = function (args) {
        return args[0] / args[1];
    };
    return Divide;
}(TotalFunctionConstraint));
var Sin = (function (_super) {
    __extends(Sin, _super);
    function Sin() {
        _super.apply(this, arguments);
    }
    Sin.prototype.getReturnValue = function (args) {
        return Math.sin(this.resolveTrigAttributes(args));
    };
    return Sin;
}(TrigConstraint));
var Cos = (function (_super) {
    __extends(Cos, _super);
    function Cos() {
        _super.apply(this, arguments);
    }
    Cos.prototype.getReturnValue = function (args) {
        return Math.cos(this.resolveTrigAttributes(args));
    };
    return Cos;
}(TrigConstraint));
var Tan = (function (_super) {
    __extends(Tan, _super);
    function Tan() {
        _super.apply(this, arguments);
    }
    Tan.prototype.getReturnValue = function (args) {
        return Math.tan(this.resolveTrigAttributes(args));
    };
    return Tan;
}(TrigConstraint));
var ASin = (function (_super) {
    __extends(ASin, _super);
    function ASin() {
        _super.apply(this, arguments);
    }
    ASin.prototype.getReturnValue = function (args) {
        return Math.asin(args[0]);
    };
    return ASin;
}(ValueOnlyConstraint));
var ACos = (function (_super) {
    __extends(ACos, _super);
    function ACos() {
        _super.apply(this, arguments);
    }
    ACos.prototype.getReturnValue = function (args) {
        return Math.acos(args[0]);
    };
    return ACos;
}(ValueOnlyConstraint));
var ATan = (function (_super) {
    __extends(ATan, _super);
    function ATan() {
        _super.apply(this, arguments);
    }
    ATan.prototype.getReturnValue = function (args) {
        return Math.atan(args[0]);
    };
    return ATan;
}(ValueOnlyConstraint));
var ATan2 = (function (_super) {
    __extends(ATan2, _super);
    function ATan2() {
        _super.apply(this, arguments);
    }
    ATan2.prototype.getReturnValue = function (args) {
        return (Math.atan2(args[0], args[1]));
    };
    ATan2.AttributeMapping = {
        "x": 0,
        "y": 1
    };
    return ATan2;
}(TotalFunctionConstraint));
//Hyperbolic Functions
var SinH = (function (_super) {
    __extends(SinH, _super);
    function SinH() {
        _super.apply(this, arguments);
    }
    SinH.prototype.sinh = function (x) {
        var y = Math.exp(x);
        return (y - 1 / y) / 2;
    };
    SinH.prototype.getReturnValue = function (args) {
        return (this.sinh(args[0]));
    };
    return SinH;
}(ValueOnlyConstraint));
var CosH = (function (_super) {
    __extends(CosH, _super);
    function CosH() {
        _super.apply(this, arguments);
    }
    CosH.prototype.cosh = function (x) {
        var y = Math.exp(x);
        return (y + 1 / y) / 2;
    };
    CosH.prototype.getReturnValue = function (args) {
        return (this.cosh(args[0]));
    };
    return CosH;
}(ValueOnlyConstraint));
var TanH = (function (_super) {
    __extends(TanH, _super);
    function TanH() {
        _super.apply(this, arguments);
    }
    TanH.prototype.tanh = function (x) {
        if (x === Infinity) {
            return 1;
        }
        else if (x === -Infinity) {
            return -1;
        }
        else {
            var y = Math.exp(2 * x);
            return (y - 1) / (y + 1);
        }
    };
    TanH.prototype.getReturnValue = function (args) {
        return (this.tanh(args[0]));
    };
    return TanH;
}(ValueOnlyConstraint));
//Inverse Hyperbolic
var ASinH = (function (_super) {
    __extends(ASinH, _super);
    function ASinH() {
        _super.apply(this, arguments);
    }
    ASinH.prototype.asinh = function (x) {
        if (x === -Infinity) {
            return x;
        }
        else {
            return Math.log(x + Math.sqrt(x * x + 1));
        }
    };
    ASinH.prototype.getReturnValue = function (args) {
        return this.asinh(args[0]);
    };
    return ASinH;
}(ValueOnlyConstraint));
var ACosH = (function (_super) {
    __extends(ACosH, _super);
    function ACosH() {
        _super.apply(this, arguments);
    }
    ACosH.prototype.acosh = function (x) {
        //How do we handle number outside of range in Eve? 
        if (x < 1) {
            return NaN;
        }
        return Math.log(x + Math.sqrt(x * x - 1));
    };
    ACosH.prototype.getReturnValue = function (args) {
        return this.acosh(args[0]);
    };
    return ACosH;
}(ValueOnlyConstraint));
var ATanH = (function (_super) {
    __extends(ATanH, _super);
    function ATanH() {
        _super.apply(this, arguments);
    }
    ATanH.prototype.atanh = function (x) {
        //How do we handle number outside of range in Eve? 
        if (Math.abs(x) > 1) {
            return NaN;
        }
        return Math.log((1 + x) / (1 - x)) / 2;
    };
    ATanH.prototype.getReturnValue = function (args) {
        return this.atanh(args[0]);
    };
    return ATanH;
}(ValueOnlyConstraint));
var Log = (function (_super) {
    __extends(Log, _super);
    function Log() {
        _super.apply(this, arguments);
    }
    Log.prototype.getReturnValue = function (args) {
        var baselog = 1;
        if (!(isNaN(args[1]))) {
            baselog = Math.log(args[1]);
        }
        return (Math.log(args[0]) / baselog);
    };
    Log.AttributeMapping = {
        "value": 0,
        "base": 1
    };
    return Log;
}(TotalFunctionConstraint));
var Exp = (function (_super) {
    __extends(Exp, _super);
    function Exp() {
        _super.apply(this, arguments);
    }
    Exp.prototype.getReturnValue = function (args) {
        return (Math.exp(args[0]));
    };
    return Exp;
}(ValueOnlyConstraint));
var Pow = (function (_super) {
    __extends(Pow, _super);
    function Pow() {
        _super.apply(this, arguments);
    }
    Pow.prototype.getReturnValue = function (args) {
        return Math.pow(args[0], args[1]);
    };
    Pow.AttributeMapping = {
        "value": 0,
        "by": 1,
    };
    return Pow;
}(TotalFunctionConstraint));
var Mod = (function (_super) {
    __extends(Mod, _super);
    function Mod() {
        _super.apply(this, arguments);
    }
    Mod.prototype.getReturnValue = function (args) {
        return args[0] % args[1];
    };
    Mod.AttributeMapping = {
        "value": 0,
        "by": 1,
    };
    return Mod;
}(TotalFunctionConstraint));
var Abs = (function (_super) {
    __extends(Abs, _super);
    function Abs() {
        _super.apply(this, arguments);
    }
    Abs.prototype.getReturnValue = function (args) {
        return Math.abs(args[0]);
    };
    return Abs;
}(ValueOnlyConstraint));
var Floor = (function (_super) {
    __extends(Floor, _super);
    function Floor() {
        _super.apply(this, arguments);
    }
    Floor.prototype.getReturnValue = function (args) {
        return Math.floor(args[0]);
    };
    return Floor;
}(ValueOnlyConstraint));
var Ceiling = (function (_super) {
    __extends(Ceiling, _super);
    function Ceiling() {
        _super.apply(this, arguments);
    }
    Ceiling.prototype.getReturnValue = function (args) {
        return Math.ceil(args[0]);
    };
    return Ceiling;
}(ValueOnlyConstraint));
var Random = (function (_super) {
    __extends(Random, _super);
    function Random() {
        _super.apply(this, arguments);
    }
    Random.prototype.getReturnValue = function (args) {
        var seed = args[0];
        var found = Random.cache[seed];
        if (found)
            return found;
        return Random.cache[seed] = Math.random();
    };
    Random.AttributeMapping = {
        "seed": 0,
    };
    Random.cache = {};
    return Random;
}(TotalFunctionConstraint));
var Gaussian = (function (_super) {
    __extends(Gaussian, _super);
    function Gaussian() {
        _super.apply(this, arguments);
    }
    Gaussian.prototype.getReturnValue = function (args) {
        var seed = args[0], sigma = args[1], mu = args[2];
        if (sigma === undefined)
            sigma = 1.0;
        if (mu === undefined)
            mu = 0.0;
        var found = Gaussian.cache[seed];
        if (found)
            return found;
        var u1 = Math.random();
        var u2 = Math.random();
        var z0 = Math.sqrt(-2.0 * Math.log(u1)) * Math.cos(Math.PI * 2 * u2);
        var key = "" + seed + sigma + mu;
        var res = z0 * sigma + mu;
        Gaussian.cache[key] = res;
        return res;
    };
    Gaussian.AttributeMapping = {
        "seed": 0,
        "σ": 1,
        "μ": 2
    };
    Gaussian.cache = {};
    return Gaussian;
}(TotalFunctionConstraint));
var Round = (function (_super) {
    __extends(Round, _super);
    function Round() {
        _super.apply(this, arguments);
    }
    Round.prototype.getReturnValue = function (args) {
        return Math.round(args[0]);
    };
    return Round;
}(ValueOnlyConstraint));
var ToFixed = (function (_super) {
    __extends(ToFixed, _super);
    function ToFixed() {
        _super.apply(this, arguments);
    }
    ToFixed.prototype.getReturnValue = function (args) {
        return args[0].toFixed(args[1]);
    };
    ToFixed.AttributeMapping = {
        "value": 0,
        "places": 1,
    };
    return ToFixed;
}(TotalFunctionConstraint));
var Range = (function (_super) {
    __extends(Range, _super);
    function Range() {
        _super.apply(this, arguments);
    }
    Range.prototype.resolveProposal = function (proposal, prefix) {
        var args = this.resolve(prefix).args;
        var from = args[0], to = args[1], increment = args[2];
        increment = increment || 1;
        var results = [];
        if (from <= to) {
            for (var val = from; val <= to; val += increment) {
                results.push(val);
            }
        }
        else {
            for (var val = from; val >= to; val += increment) {
                results.push(val);
            }
        }
        return results;
    };
    Range.prototype.test = function (prefix) {
        var _a = this.resolve(prefix), args = _a.args, returns = _a.returns;
        var from = args[0], to = args[1], increment = args[2];
        increment = increment || 1;
        var val = returns[0];
        var member = from <= val && val <= to &&
            ((val - from) % increment) == 0;
        return member;
    };
    Range.prototype.getProposal = function (tripleIndex, proposed, prefix) {
        var args = this.resolve(prefix).args;
        var from = args[0], to = args[1], increment = args[2];
        increment = args[2] || 1;
        var proposal = this.proposalObject;
        proposal.providing = proposed;
        if (from <= to && increment < 0) {
            proposal.cardinality = 0;
            return proposal;
        }
        else if (from > to && increment > 0) {
            proposal.cardinality = 0;
            return proposal;
        }
        proposal.cardinality = Math.ceil(Math.abs((to - from + 1) / increment));
        return proposal;
    };
    Range.AttributeMapping = {
        "from": 0,
        "to": 1,
        "increment": 2,
    };
    return Range;
}(join_1.Constraint));
//Constants
var PI = (function (_super) {
    __extends(PI, _super);
    function PI() {
        _super.apply(this, arguments);
    }
    PI.prototype.getReturnValue = function (args) {
        return Math.PI;
    };
    return PI;
}(TotalFunctionConstraint));
var E = (function (_super) {
    __extends(E, _super);
    function E() {
        _super.apply(this, arguments);
    }
    E.prototype.getReturnValue = function (args) {
        return Math.E;
    };
    return E;
}(TotalFunctionConstraint));
var LN2 = (function (_super) {
    __extends(LN2, _super);
    function LN2() {
        _super.apply(this, arguments);
    }
    LN2.prototype.getReturnValue = function (args) {
        return Math.LN2;
    };
    return LN2;
}(TotalFunctionConstraint));
var LN10 = (function (_super) {
    __extends(LN10, _super);
    function LN10() {
        _super.apply(this, arguments);
    }
    LN10.prototype.getReturnValue = function (args) {
        return Math.LN10;
    };
    return LN10;
}(TotalFunctionConstraint));
var LOG2E = (function (_super) {
    __extends(LOG2E, _super);
    function LOG2E() {
        _super.apply(this, arguments);
    }
    LOG2E.prototype.getReturnValue = function (args) {
        return Math.LOG2E;
    };
    return LOG2E;
}(TotalFunctionConstraint));
var LOG10E = (function (_super) {
    __extends(LOG10E, _super);
    function LOG10E() {
        _super.apply(this, arguments);
    }
    LOG10E.prototype.getReturnValue = function (args) {
        return Math.LOG10E;
    };
    return LOG10E;
}(TotalFunctionConstraint));
var SQRT1_2 = (function (_super) {
    __extends(SQRT1_2, _super);
    function SQRT1_2() {
        _super.apply(this, arguments);
    }
    SQRT1_2.prototype.getReturnValue = function (args) {
        return Math.SQRT1_2;
    };
    return SQRT1_2;
}(TotalFunctionConstraint));
var SQRT2 = (function (_super) {
    __extends(SQRT2, _super);
    function SQRT2() {
        _super.apply(this, arguments);
    }
    SQRT2.prototype.getReturnValue = function (args) {
        return Math.SQRT2;
    };
    return SQRT2;
}(TotalFunctionConstraint));
providers.provide("+", Add);
providers.provide("-", Subtract);
providers.provide("*", Multiply);
providers.provide("/", Divide);
providers.provide("log", Log);
providers.provide("exp", Exp);
//Trig and Inverse Trig
providers.provide("sin", Sin);
providers.provide("cos", Cos);
providers.provide("tan", Tan);
providers.provide("asin", ASin);
providers.provide("acos", ACos);
providers.provide("atan", ATan);
providers.provide("atan2", ATan2);
//Hyperbolic Functions.
providers.provide("sinh", SinH);
providers.provide("cosh", CosH);
providers.provide("tanh", TanH);
providers.provide("asinh", ASinH);
providers.provide("acosh", ACosH);
providers.provide("atanh", ATanH);
providers.provide("floor", Floor);
providers.provide("ceiling", Ceiling);
providers.provide("abs", Abs);
providers.provide("mod", Mod);
providers.provide("pow", Pow);
providers.provide("random", Random);
providers.provide("range", Range);
providers.provide("round", Round);
providers.provide("gaussian", Gaussian);
providers.provide("to-fixed", ToFixed);
//Constants
providers.provide("pi", PI);
providers.provide("e", E);
providers.provide("ln2", LN2);
providers.provide("ln10", LN10);
providers.provide("log2e", LOG2E);
providers.provide("log10e", LOG10E);
providers.provide("sqrt1/2", SQRT1_2);
providers.provide("sqrt2", SQRT2);
//# sourceMappingURL=math.js.map