"use strict";
var test = require("tape");
var shared_functions_1 = require("./shared_functions");
// Test Constants
// Would this fail on different architectures than mine? Not according to dcmumentation Maybe we Should
// flush out that implicit assumption.
var constants_list = [
    { "Expression": "pi[]", "Value": "3.141592653589793" },
    { "Expression": "e[]", "Value": "2.718281828459045" },
    { "Expression": "ln2[]", "Value": "0.6931471805599453" },
    { "Expression": "log2e[]", "Value": "1.4426950408889634" },
    { "Expression": "log10e[]", "Value": "0.4342944819032518" },
    { "Expression": "sqrt1/2[]", "Value": "0.7071067811865476" },
    { "Expression": "sqrt2[]", "Value": "1.4142135623730951" }
];
shared_functions_1.testSingleExpressionByList(constants_list);
// Test Power Function
var pow_list = [
    { "Expression": "pow[ value:2 by:3 ]", "Value": "8" },
    { "Expression": "pow[ value:9 by:1 / 2 ]", "Value": "3" }];
shared_functions_1.testSingleExpressionByList(pow_list);
// Test Log Function
var log_list = [
    { "Expression": "log[ value: e[] ]", "Value": "1" },
    { "Expression": "log[ value: 10 base: 10 ]", "Value": "1" }];
shared_functions_1.testSingleExpressionByList(log_list);
// Test Exp Function
var exp_list = [
    { "Expression": "exp[ value: 1 ]", "Value": "2.718281828459045" }];
shared_functions_1.testSingleExpressionByList(exp_list);
// Test Trig Functions
var trig_list = [
    { "Expression": "sin[ radians: 1 ]", "Value": "0.8414709848078965" },
    { "Expression": "cos[ radians: 1 ]", "Value": "0.5403023058681398" },
    { "Expression": "tan[ radians: 1 ]", "Value": "1.5574077246549023" }
];
shared_functions_1.testSingleExpressionByList(trig_list);
test("Should be able to use the sin function with degrees and radians", function (assert) {
    var expected = {
        insert: [
            ["a", "tag", "div"],
            ["a", "text", "1"],
            ["b", "tag", "div"],
            ["b", "text", "0.9999996829318346"],
        ],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    Now consider this:\n\n    ~~~\n    search\n      y = sin[degrees: 90]\n      x = sin[radians: 3.14 / 2]\n\n    bind @browser\n      [#div text: y]\n      [#div text: x]\n    ~~~\n  ");
    assert.end();
});
test("Should be able to use the cos function with degrees and radians", function (assert) {
    var expected = {
        insert: [
            ["a", "tag", "div"],
            ["a", "text", "1"],
            ["b", "tag", "div"],
            ["b", "text", "-0.9999987317275395"],
        ],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    Now consider this:\n\n    ~~~\n    search\n      y = cos[degrees: 0]\n      x = cos[radians: 3.14]\n\n    bind @browser\n      [#div text: y]\n      [#div text: x]\n    ~~~\n  ");
    assert.end();
});
test("Should be able to use the tan function with degrees and radians", function (assert) {
    var expected = {
        insert: [
            ["a", "tag", "div"],
            ["a", "text", "0.5773502691896257"],
            ["b", "tag", "div"],
            ["b", "text", "0.5463024898437905"],
        ],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    Now consider this:\n\n    ~~~\n    search\n      y = tan[degrees: 30]\n      x = tan[radians: 0.5]\n\n    bind @browser\n      [#div text: y]\n      [#div text: x]\n    ~~~\n  ");
    assert.end();
});
// Test inverse Trig
var atrig_list = [
    { "Expression": "asin[ value: 0.8414709848078965 ]", "Value": "1" },
    // Does Eve need an implicit round under the hood? The below should be 1
    { "Expression": "acos[ value: 0.5403023058681398 ]", "Value": "0.9999999999999999" },
    { "Expression": "atan[ value: 1.5574077246549023 ]", "Value": "1" },
];
shared_functions_1.testSingleExpressionByList(atrig_list);
// Test Hyperbolic Functions
var hyp_list = [
    { "Expression": "sinh[ value: 1 ]", "Value": "1.1752011936438014" },
    { "Expression": "cosh[ value: 1 ]", "Value": "1.5430806348152437" },
    { "Expression": "tanh[ value: 1 ]", "Value": "0.7615941559557649" },
];
shared_functions_1.testSingleExpressionByList(hyp_list);
// Test Inverse Hyperbolic Functions
var ahyp_list = [
    { "Expression": "asinh[ value: 1.1752011936438014 ]", "Value": "1" },
    { "Expression": "acosh[ value: 1.5430806348152437 ]", "Value": "1" },
    { "Expression": "atanh[ value: 0.7615941559557649 ]", "Value": "0.9999999999999999" },
];
shared_functions_1.testSingleExpressionByList(ahyp_list);
test("Range and function within function", function (assert) {
    var expected = {
        insert: [
            ["a", "tag", "div"],
            ["a", "text", "1"],
            ["b", "tag", "div"],
            ["b", "text", "2"],
            ["c", "tag", "div"],
            ["c", "text", "3"],
        ],
        remove: [],
    };
    shared_functions_1.evaluate(assert, expected, "\n    Now consider this:\n\n    ~~~\n    search\n      x = range[from:1 to: pi[] increment: 1 ]\n\n    bind @browser\n      [#div text:x]\n    ~~~\n  ");
    assert.end();
});
// Test Floor Function
var floor_list = [
    { "Expression": "floor[ value: 1.0000000000000001 ]", "Value": "1" },
    { "Expression": "floor[ value: 1.999999999999999 ]", "Value": "1" },
];
shared_functions_1.testSingleExpressionByList(floor_list);
// Test Ceiling Function
var ceiling_list = [
    { "Expression": "ceiling[ value: 1.000000000000001 ]", "Value": "2" },
    { "Expression": "ceiling[ value: 1.999999999999999 ]", "Value": "2" },
];
shared_functions_1.testSingleExpressionByList(ceiling_list);
// Test ABS Function
var abs_list = [
    { "Expression": "abs[ value: -1 ]", "Value": "1" },
    { "Expression": "abs[ value: 1 ]", "Value": "1" },
];
shared_functions_1.testSingleExpressionByList(abs_list);
// Test Mod Function
var mod_list = [
    { "Expression": "mod[ value: 7 by: 3]", "Value": "1" },
    { "Expression": "mod[ value: 6 by: 3]", "Value": "0" },
];
shared_functions_1.testSingleExpressionByList(mod_list);
// Test Round Function
var round_list = [
    { "Expression": "round[ value: 1.49999999999999 ]", "Value": "1" },
    { "Expression": "round[ value: 1.5 ]", "Value": "2" }
];
shared_functions_1.testSingleExpressionByList(round_list);
// Test Round Function
var toFixed_list = [
    { "Expression": "to-fixed[ value: 1.499 places:2 ]", "Value": "1.50" },
];
shared_functions_1.testSingleExpressionByList(toFixed_list);
//# sourceMappingURL=math.js.map